use serde::{Deserialize, Serialize};
use sqlx::{postgres::PgPool, FromRow, Type};
use time::OffsetDateTime;
use uuid::Uuid;

use crate::error::AppError;

#[derive(Debug, Serialize, Deserialize, FromRow, Clone, PartialEq, Type)]
#[sqlx(rename_all = "snake_case")]
#[serde(rename_all = "snake_case")]
#[sqlx(type_name = "text")] // Tells sqlx to treat this enum as text
pub enum FileType {
    File,
    Directory,
}

#[derive(Debug, Serialize, Deserialize, FromRow)]
pub struct File {
    pub id: Uuid,
    pub name: String,
    pub file_type: FileType,
    pub mime_type: Option<String>,
    pub size: i64,
    pub owner_id: Uuid,
    pub parent_id: Option<Uuid>,
    pub encryption_key: Option<String>, // Encrypted with master key
    pub encryption_iv: Option<String>,  // Initialization vector for AES-GCM
    pub created_at: OffsetDateTime,
    pub updated_at: OffsetDateTime,
}

#[derive(Debug, Deserialize)]
pub struct CreateFileDto {
    pub name: String,
    pub file_type: FileType,
    pub mime_type: Option<String>,
    pub size: i64,
    pub parent_id: Option<Uuid>,
}

#[derive(Debug, Deserialize)]
pub struct CreateDirectoryDto {
    pub name: String,
    pub parent_id: Option<Uuid>,
}

impl File {
    pub async fn create_file(
        pool: &PgPool,
        dto: CreateFileDto,
        owner_id: Uuid,
        encryption_key: Option<String>,
        encryption_iv: Option<String>,
    ) -> Result<Self, AppError> {
        if let Some(parent_id) = dto.parent_id {
            let parent = Self::find_by_id(pool, parent_id).await?;
            if parent.file_type != FileType::Directory {
                return Err(AppError::NotADirectory);
            }
            if parent.owner_id != owner_id {
                // TODO: Check if user has write permission through sharing
                return Err(AppError::AccessDenied);
            }
        }

        let existing_file = sqlx::query!("SELECT id FROM files WHERE name = $1 AND parent_id IS NOT DISTINCT FROM $2 AND owner_id = $3",
            dto.name, dto.parent_id, owner_id)
            .fetch_optional(pool)
            .await?;

        if existing_file.is_some() {
            return Err(AppError::FileAlreadyExists);
        }

        let file = sqlx::query_as!(File,
            r#"INSERT INTO files (id, name, file_type, mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at)
            VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)
            RETURNING id, name, file_type as "file_type: FileType", mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at"#,
            Uuid::new_v4(),
            dto.name,
            dto.file_type as FileType,
            dto.mime_type,
            dto.size,
            owner_id,
            dto.parent_id,
            encryption_key,
            encryption_iv,
            OffsetDateTime::now_utc(),
            OffsetDateTime::now_utc()
        )
        .fetch_one(pool)
        .await?;

        Ok(file)
    }

    pub async fn create_directory(
        pool: &PgPool,
        dto: CreateDirectoryDto,
        owner_id: Uuid,
    ) -> Result<Self, AppError> {
        if let Some(parent_id) = dto.parent_id {
            let parent = Self::find_by_id(pool, parent_id).await?;
            if parent.file_type != FileType::Directory {
                return Err(AppError::NotADirectory);
            }
            if parent.owner_id != owner_id {
                // TODO: Check if user has write permission through sharing
                return Err(AppError::AccessDenied);
            }
        }

        let existing_dir = sqlx::query!("SELECT id FROM files WHERE name = $1 AND parent_id IS NOT DISTINCT FROM $2 AND owner_id = $3 AND file_type = 'directory'",
            dto.name, dto.parent_id, owner_id)
            .fetch_optional(pool)
            .await?;

        if existing_dir.is_some() {
            return Err(AppError::DirectoryAlreadyExists);
        }

        let directory = sqlx::query_as!(File,
            r#"INSERT INTO files (id, name, file_type, mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at)
            VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)
            RETURNING id, name, file_type as "file_type: FileType", mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at"#,
            Uuid::new_v4(),
            dto.name,
            FileType::Directory,
            None as Option<String>,
            0i64,
            owner_id,
            dto.parent_id,
            None as Option<String>,
            None as Option<String>,
            OffsetDateTime::now_utc(),
            OffsetDateTime::now_utc()
        )
        .fetch_one(pool)
        .await?;

        Ok(directory)
    }

    pub async fn find_by_id(pool: &PgPool, id: Uuid) -> Result<Self, AppError> {
        let file = sqlx::query_as!(File,
            r#"SELECT id, name, file_type as "file_type: FileType", mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at
            FROM files WHERE id = $1"#,
            id
        )
        .fetch_optional(pool)
        .await?
        .ok_or(AppError::FileNotFound)?;

        Ok(file)
    }

    pub async fn list_directory(
        pool: &PgPool,
        directory_id: Option<Uuid>,
        user_id: Uuid,
    ) -> Result<Vec<Self>, AppError> {
        let files = if let Some(dir_id) = directory_id {
            let directory = Self::find_by_id(pool, dir_id).await?;
            if directory.file_type != FileType::Directory {
                return Err(AppError::NotADirectory);
            }
            
            if directory.owner_id != user_id {
                return Err(AppError::AccessDenied);
            }
            
            sqlx::query_as!(File,
                r#"SELECT id, name, file_type as "file_type: FileType", mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at
                FROM files WHERE parent_id = $1 ORDER BY file_type, name"#,
                dir_id
            )
            .fetch_all(pool)
            .await?
        } else {
            sqlx::query_as!(File,
                r#"SELECT id, name, file_type as "file_type: FileType", mime_type, size, owner_id, parent_id, encryption_key, encryption_iv, created_at, updated_at
                FROM files WHERE parent_id IS NULL AND owner_id = $1 ORDER BY file_type, name"#,
                user_id
            )
            .fetch_all(pool)
            .await?
        };

        Ok(files)
    }

    pub async fn delete(
        pool: &PgPool,
        id: Uuid,
        user_id: Uuid,
    ) -> Result<(), AppError> {
        let file = Self::find_by_id(pool, id).await?;
        
        if file.owner_id != user_id {
            return Err(AppError::AccessDenied);
        }
        
        if file.file_type == FileType::Directory {
            let files_in_dir = Self::list_directory(pool, Some(id), user_id).await?;
            
            for file in files_in_dir {
                Self::delete(pool, file.id, user_id).await?;
            }
        }
        
        sqlx::query!("DELETE FROM files WHERE id = $1", id)
            .execute(pool)
            .await?;
            
        Ok(())
    }

    pub async fn get_file_path(pool: &PgPool, id: Uuid) -> Result<String, AppError> {
        let file = Self::find_by_id(pool, id).await?;
        
        let mut path_parts = vec![file.name.clone()];
        let mut current_parent_id = file.parent_id;
        
        let mut depth = 0;
        const MAX_DEPTH: usize = 100;
        
        while let Some(parent_id) = current_parent_id {
            if depth >= MAX_DEPTH {
                return Err(AppError::PathTooDeep);
            }
            
            let parent = Self::find_by_id(pool, parent_id).await?;
            path_parts.push(parent.name.clone());
            current_parent_id = parent.parent_id;
            depth += 1;
        }
        
        path_parts.reverse();
        Ok(path_parts.join("/"))
    }
}